@@ -32,9 +32,9 @@
 #include <KIO/StoredTransferJob>
 #include <KJobWidgets>
 #include <KNotification>
-#include <phonon/mediaobject.h>
-#include <phonon/audiooutput.h>
-#include <phonon/volumefadereffect.h>
+#include <phonon/MediaObject>
+#include <phonon/AudioOutput>
+#include <phonon/VolumeFaderEffect>
 
 #include <QLocale>
 #include <QTimer>
@@ -44,6 +44,7 @@
 #include <QUrl>
 #include <QTextBrowser>
 #include <QTemporaryFile>
+#include <QRegExp>
 
 using namespace KAlarmCal;
 
@@ -84,7 +85,7 @@ MessageDisplayHelper::MessageDisplayHelper(MessageDisplay* parent, const KAEvent
     , mDefaultDeferMinutes(event.deferDefaultMinutes())
     , mAlarmType(alarm.type())
     , mAction(event.actionSubType())
-    , mAkonadiItemId(event.akonadiItemId())
+    , mEmailId(event.emailId())
     , mCommandError(event.commandError())
     , mAudioRepeatPause(event.repeatSoundPause())
     , mConfirmAck(event.confirmAck())
@@ -139,7 +140,7 @@ MessageDisplayHelper::MessageDisplayHelper(MessageDisplay* parent, const KAEvent
     , mEventId(event)
     , mAlarmType(KAAlarm::MAIN_ALARM)
     , mAction(event.actionSubType())
-    , mAkonadiItemId(-1)
+    , mEmailId(-1)
     , mCommandError(KAEvent::CMD_NO_ERROR)
     , mErrorMsgs(errmsgs)
     , mDontShowAgain(dontShowAgain)
@@ -174,7 +175,7 @@ MessageDisplayHelper::~MessageDisplayHelper()
 {
     qCDebug(KALARM_LOG) << "~MessageDisplayHelper()" << mEventId;
     if (mAudioOwner == this  &&  !mAudioPlayer.isNull())
-        delete mAudioPlayer;
+        mAudioPlayer->stop();   // mAudioPlayer will delete once stopped
     if (mAudioOwner == this)
         mAudioOwner = nullptr;
     // If the audio thread is destroyed while still running, it will crash.
@@ -210,7 +211,7 @@ void MessageDisplayHelper::initTexts()
             // enclosing "Reminder".
             QString s = i18nc("@info", "Reminder");
             QRegExp re(QStringLiteral("^(<[^>]+>)*"));  // search for HTML tag "<...>"
-            re.indexIn(s);
+            (void) re.indexIn(s);
             // Prefix the time, plus a newline, to "Reminder", inside any HTML tags.
             s.insert(re.matchedLength(), mTexts.time + QLatin1String("<br/>"));
             mTexts.timeFull = s;
@@ -271,7 +272,7 @@ void MessageDisplayHelper::initTexts()
                             case File::TextFormatted:
                             {
                                 QTextBrowser browser;
-                                browser.setSource(QUrl::fromLocalFile(mTempFile->fileName()));   //krazy:exclude=qclasses
+                                browser.setSource(QUrl::fromLocalFile(mTempFile->fileName()));
                                 mTexts.message = browser.toHtml();
                                 delete mTempFile;
                                 mTempFile = nullptr;
@@ -605,7 +606,7 @@ bool MessageDisplayHelper::saveProperties(KConfigGroup& config)
         config.writeEntry("DeferMins", mDefaultDeferMinutes);
         config.writeEntry("NoDefer", mNoDefer);
         config.writeEntry("NoPostAction", mNoPostAction);
-        config.writeEntry("AkonadiItemId", mAkonadiItemId);
+        config.writeEntry("EmailId", mEmailId);
         config.writeEntry("CmdErr", static_cast<int>(mCommandError));
         config.writeEntry("DontShowAgain", mDontShowAgain);
         return true;
@@ -681,7 +682,7 @@ bool MessageDisplayHelper::readPropertyValues(const KConfigGroup& config)
     mDefaultDeferMinutes = config.readEntry("DeferMins", 0);
     mNoDefer             = config.readEntry("NoDefer", false);
     mNoPostAction        = config.readEntry("NoPostAction", true);
-    mAkonadiItemId       = config.readEntry("AkonadiItemId", QVariant(QVariant::LongLong)).toLongLong();
+    mEmailId             = config.readEntry("EmailId", QVariant(QVariant::LongLong)).toLongLong();
     mCommandError        = KAEvent::CmdErrType(config.readEntry("CmdErr", static_cast<int>(KAEvent::CMD_NO_ERROR)));
     mDontShowAgain       = config.readEntry("DontShowAgain", QString());
     mShowEdit            = false;
@@ -883,7 +884,7 @@ void MessageDisplayHelper::startAudio()
         connect(audioPlayer, &AudioPlayer::readyToPlay, this, &MessageDisplayHelper::playReady);
         connect(audioThread, &QThread::finished, this, &MessageDisplayHelper::playFinished);
         if (mSilenceButton)
-            connect(mSilenceButton, &QAbstractButton::clicked, audioThread, &QThread::quit);
+            connect(mSilenceButton, &QAbstractButton::clicked, this, &MessageDisplayHelper::stopAudioPlay);
 
         // Notify after creating mAudioPlayer, so that isAudioPlaying() will
         // return the correct value.
@@ -903,23 +904,11 @@ bool MessageDisplayHelper::isAudioPlaying()
 /******************************************************************************
 * Stop audio playback.
 */
-void MessageDisplayHelper::stopAudio(bool wait)
+void MessageDisplayHelper::stopAudio()
 {
     qCDebug(KALARM_LOG) << "MessageDisplayHelper::stopAudio";
-    delete mAudioPlayer.data();
-    if (mAudioThread)
-    {
-        // Quit the audio thread and wait for thread completion and tidy up.
-        mAudioThread->quit();       // stop playing and tidy up
-        mAudioThread->wait(3000);   // wait for run() to exit (timeout 3 seconds)
-        if (!mAudioThread->isFinished())
-        {
-            // Something has gone wrong - forcibly kill the thread
-            mAudioThread->terminate();
-            if (wait)
-                mAudioThread->wait();
-        }
-    }
+    if (mAudioPlayer)
+        mAudioPlayer->stop();
 }
 
 /******************************************************************************
@@ -971,7 +960,6 @@ AudioPlayer::AudioPlayer(const QString& audioFile, float volume, float fadeVolum
     , mFadeVolume(fadeVolume)
     , mFadeSeconds(fadeSeconds)
     , mRepeatPause(repeatPause)
-    , mAudioObject(nullptr)
 {
 }
 
@@ -982,8 +970,17 @@ AudioPlayer::AudioPlayer(const QString& audioFile, float volume, float fadeVolum
 AudioPlayer::~AudioPlayer()
 {
     qCDebug(KALARM_LOG) << "MessageDisplayHelper::~AudioPlayer";
+    mMutex.lock();
+    mPath.disconnect();
+    if (mFader)
+    {
+        mPath.removeEffect(mFader);
+        delete mFader;
+        mFader = nullptr;
+    }
     delete mAudioObject;
     mAudioObject = nullptr;
+    mMutex.unlock();
     // Notify after deleting mAudioPlayer, so that isAudioPlaying() will
     // return the correct value.
     QTimer::singleShot(0, theApp(), &KAlarmApp::notifyAudioStopped);   //NOLINT(clang-analyzer-cplusplus.NewDeleteLeaks)
@@ -1000,7 +997,7 @@ void AudioPlayer::execute()
         mMutex.unlock();
         return;
     }
-    qCDebug(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer::run:" << QThread::currentThread() << mFile;
+    qCDebug(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer::execute:" << QThread::currentThread() << mFile;
     const QString audioFile = mFile;
     const QUrl url = QUrl::fromUserInput(mFile);
     mFile = url.isLocalFile() ? url.toLocalFile() : url.toString();
@@ -1009,25 +1006,34 @@ void AudioPlayer::execute()
     {
         mError = xi18nc("@info", "Cannot open audio file: <filename>%1</filename>", audioFile);
         mMutex.unlock();
-        qCCritical(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer::run: Open failure:" << audioFile;
+        qCCritical(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer::execute: Open failure:" << audioFile;
         return;
     }
-    mAudioObject = new Phonon::MediaObject();
+    mAudioObject = new Phonon::MediaObject(this);
     mAudioObject->setCurrentSource(source);
     mAudioObject->setTransitionTime(100);   // workaround to prevent clipping of end of files in Xine backend
-    auto output = new Phonon::AudioOutput(Phonon::NotificationCategory, mAudioObject);   //NOLINT(clang-analyzer-cplusplus.NewDeleteLeaks)
+    auto output = new Phonon::AudioOutput(Phonon::NotificationCategory, this);   //NOLINT(clang-analyzer-cplusplus.NewDeleteLeaks)
     mPath = Phonon::createPath(mAudioObject, output);
     if (mVolume >= 0  ||  mFadeVolume >= 0)
     {
-        const float vol = (mVolume >= 0) ? mVolume : output->volume();
-        const float maxvol = qMax(vol, mFadeVolume);
-        output->setVolume(maxvol);
+        if (mVolume >= 0)
+            output->setVolume(mVolume);
         if (mFadeVolume >= 0  &&  mFadeSeconds > 0)
         {
-            auto fader = new Phonon::VolumeFaderEffect(mAudioObject);
-            fader->setVolume(mFadeVolume / maxvol);
-            fader->fadeTo(mVolume / maxvol, mFadeSeconds * 1000);
-            mPath.insertEffect(fader);
+            mFader = new Phonon::VolumeFaderEffect(this);
+            if (!mFader->isValid())
+            {
+                // The current Phonon backend doesn't support fading.
+                qCWarning(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer: Current Phonon backend does not support fading";
+                delete mFader;
+                mFader = nullptr;
+            }
+            else
+            {
+                if (mVolume < 0)
+                    mVolume = output->volume();
+                mPath.insertEffect(mFader);
+            }
         }
     }
     connect(mAudioObject, &Phonon::MediaObject::stateChanged, this, &AudioPlayer::playStateChanged, Qt::DirectConnection);
@@ -1064,7 +1070,7 @@ void AudioPlayer::checkAudioPlay()
             {
                 // Play has completed
                 mMutex.unlock();
-                stopPlay();
+                stop();
                 return;
             }
             if (mRepeatPause > 0)
@@ -1081,6 +1087,11 @@ void AudioPlayer::checkAudioPlay()
 
     // Start playing the file, either for the first time or again
     qCDebug(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer::checkAudioPlay: start";
+    if (mFader)
+    {
+        mFader->setVolume(mFadeVolume);
+        mFader->fadeTo(mVolume, mFadeSeconds * 1000);
+    }
     mAudioObject->play();
     mMutex.unlock();
 }
@@ -1091,16 +1102,42 @@ void AudioPlayer::checkAudioPlay()
 */
 void AudioPlayer::playStateChanged(Phonon::State newState)
 {
-    if (newState == Phonon::ErrorState)
+    qCDebug(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer::playStateChanged:" << newState;
+    switch (newState)
     {
-        QMutexLocker locker(&mMutex);
-        const QString err = mAudioObject->errorString();
-        if (!err.isEmpty())
+        case Phonon::StoppedState:
+            mMutex.lock();
+            if (mStopping)
+            {
+                mPath.disconnect();
+                if (mFader)
+                {
+                    // Delete fader from the audio thread, in case it is still active.
+                    // because its timers can't be stopped from another thread.
+                    mPath.removeEffect(mFader);
+                    delete mFader;
+                    mFader = nullptr;
+                }
+                deleteLater();
+            }
+            mMutex.unlock();
+            break;
+
+        case Phonon::ErrorState:
         {
-            qCCritical(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer::playStateChanged: Play failure:" << mFile << ":" << err;
-            mError = xi18nc("@info", "<para>Error playing audio file: <filename>%1</filename></para><para>%2</para>", mFile, err);
-            exit(1);
+            QMutexLocker locker(&mMutex);
+            const QString err = mAudioObject->errorString();
+            if (!err.isEmpty())
+            {
+                qCCritical(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer::playStateChanged: Play failure:" << mFile << ":" << err;
+                mError = xi18nc("@info", "<para>Error playing audio file: <filename>%1</filename></para><para>%2</para>", mFile, err);
+                exit(1);
+            }
+            break;
         }
+
+        default:
+            break;
     }
 }
 
@@ -1108,20 +1145,19 @@ void AudioPlayer::playStateChanged(Phonon::State newState)
 * Called when play completes, the Silence button is clicked, or the display is
 * closed, to terminate audio access.
 */
-void AudioPlayer::stopPlay()
+void AudioPlayer::stop()
 {
+    qCDebug(KALARM_LOG) << "MessageDisplayHelper::AudioPlayer::stop";
     mMutex.lock();
+    mStopping = true;
     if (mAudioObject)
     {
-        mAudioObject->stop();
-        const QList<Phonon::Effect*> effects = mPath.effects();
-        for (int i = 0;  i < effects.count();  ++i)
+        if (mAudioObject->state() != Phonon::StoppedState)
         {
-            mPath.removeEffect(effects[i]);
-            delete effects[i];
+            mAudioObject->stop();
+            mMutex.unlock();
+            return;
         }
-        delete mAudioObject;
-        mAudioObject = nullptr;
     }
     mMutex.unlock();
     deleteLater();
